

//-----------------------------------------------------------------------------
// Window_Base
//
Window_Base.prototype.standardFontSize = function() {
    return 22;
};   

Window_Base.prototype.standardPadding = function() {
    return 0;
};

Window_Base.prototype.standardBackOpacity = function() {
    return 255;
};

Window_Base.prototype.calcTextHeight = function(textState, all) {
    var lastFontSize = this.contents.fontSize;
    var textHeight = 0;
    var lines = textState.text.slice(textState.index).split('\n');
    var maxLines = all ? lines.length : 1;

    for (var i = 0; i < maxLines; i++) {
        var maxFontSize = this.contents.fontSize;
        var regExp = /\x1b[\{\}]/g;
        for (;;) {
            var array = regExp.exec(lines[i]);
            if (array) {
                if (array[0] === '\x1b{') {
                    this.makeFontBigger();
                }
                if (array[0] === '\x1b}') {
                    this.makeFontSmaller();
                }
                if (maxFontSize < this.contents.fontSize) {
                    maxFontSize = this.contents.fontSize;
                }
            } else {
                break;
            }
        }
        //行の高さ調整
        if ($novel_mode) {
        	textHeight += maxFontSize + 24;
        } else {
        	textHeight += maxFontSize + 12;
        }
    }

    this.contents.fontSize = lastFontSize;
    return textHeight;
};

Window_Base.prototype.makeFontBigger = function() {
    if (this.contents.fontSize <= 96) {
        this.contents.fontSize += 3;
    }
};

Window_Base.prototype.makeFontSmaller = function() {
    if (this.contents.fontSize >= 24) {
        this.contents.fontSize -= 3;
    }
};

//-----------------------------------------------------------------------------
// Window_Message
//
Window_Message.prototype.initialize = function() {
    var width = this.windowWidth();
    var height = this.windowHeight();
    //var height = 180;
    var x = (Graphics.boxWidth - width) / 2;
    Window_Base.prototype.initialize.call(this, x, 0, width, height);
    this.openness = 0;
    this.initMembers();
    this.createSubWindows();
    this.updatePlacement();
};

Window_Message.prototype.initMembers = function() {
    this._background = 0;
    this._positionType = 2;
    this._waitCount = 0;
    this._faceBitmap = null;
    this._textState = null;
    this.clearFlags();
    this._name = '';
    this._portrait = '';
};

Window_Message.prototype.newPage = function(textState) {
	var portrait = '';
	
    this.contents.clear();    
    this.resetTextColor();

    this._name = '';
    textState.text = textState.text.replace(/\x1bN\<(.*)\>/gi, function() {
		this._name = arguments[1];
		return '';
	}.bind(this));
	
    //this._portrait = '';
    textState.text = textState.text.replace(/\x1bP\<(.*)\>/gi, function() {
		portrait = arguments[1];
		return '';
	}.bind(this));
	
	if (portrait == '') {
		this._portrait = '';
		$gameScreen.erasePicture(21);
	} else if (portrait == this._portrait) {
		$gameScreen.showPicture(21, "portrait" + this._portrait, 0, 360, 0, 100, 100, 255, 0);
	} else if (portrait != this._portrait) {
		this._portrait = portrait;
		$gameScreen.showPicture(21, "portrait" + this._portrait, 0, 370, 0, 100, 100, 0, 0);
		$gameScreen.movePicture(21, 0, 360, 0, 100, 100, 255, 0, 8);
	}
	
	//if (!this._portrait == '') {
	//	$gameScreen.showPicture(21, "portrait" + this._portrait, 0, 530, 0, 100, 100, 0, 0);
	//	$gameScreen.movePicture(21, 0, 520, 0, 100, 100, 255, 0, 8);
	//} else {
	//	$gameScreen.erasePicture(21);
	//}
	
    this.contents.fontSize = 18;
    if (!this._name == '') {
		var time = this._name.length
    	for (var i = 1 ; i <= time ; i++) {
    		this._name = insertStr(this._name, i * 2 -1, '\t');
    	}
    	this.contents.drawText(this._name, 40, 26, 300, 0);
    }
    
    this.resetFontSettings();
    this.clearFlags();
    this.loadMessageFace();
    textState.x = this.newLineX();
    textState.y = this.newLineY();
    textState.left = this.newLineX();
    textState.height = this.calcTextHeight(textState, false);
    this.height = $novel_mode ? 580 : 180;
};
function insertStr(str, index, insert) {
    return str.slice(0, index) + insert + str.slice(index, str.length);
}

Window_Message.prototype.update = function() {
    this.checkToNotClose();
    this.clearPortrait();    
    Window_Base.prototype.update.call(this);
    while (!this.isOpening() && !this.isClosing()) {
        if (this.updateWait()) {
            return;
        } else if (this.updateLoading()) {
            return;
        } else if (this.updateInput()) {
            return;
        } else if (this.updateMessage()) {
            return;
        } else if (this.canStart()) {
            this.startMessage();
        } else {
            this.startInput();
            return;
        }
    }
};
Window_Message.prototype.clearPortrait = function() {    
    if (this.isClosing() && !this.isOpen()) {
    	this._portrait = '';
    	$gameScreen.erasePicture(21);
    }
};

//Window_Message.prototype.setName = function(name) {
//	this.name = name;
//	return '';
//};

Window_Message.prototype.newLineX = function() {
	if ($novel_mode) {
		return 86;
	} else if (this._name == '') {
		return 32;
	} else {
		return 60;
	}
    //return this._name == '' ? 32 : 60;
};

Window_Message.prototype.newLineY = function() {
    //return $gameMessage.faceName() === '' ? 0 : 60;
    if ($novel_mode) {
    	return 86;
    } else {
    	return this._name == '' ? 18 : 52;
    }
};

Window_Message.prototype.numVisibleRows = function() { 
    return 15;
};

Window_Message.prototype.updatePlacement = function() {
    this._positionType = $gameMessage.positionType();
    this.y = this._positionType * (Graphics.boxHeight - this.height) / 2;
    //this.y -= 20 * (this._positionType - 1);
    
    this.y += $gameVariables.value(51);
    if ($novel_mode) {
    	this.y += 22;
    }
    
    this._goldWindow.y = this.y > 0 ? 0 : Graphics.boxHeight - this._goldWindow.height;
};

Window_Message.prototype.checkToNotClose = function() {
    if (this.isClosing() && this.isOpen()) {
        if (this.doesContinue()) {
            this.open();
        }
    } else {    
		//$gameScreen.erasePicture(21);
    
    }
};

Window_Message.prototype.terminateMessage = function() {
	//$gameScreen.erasePicture(21);
    this.close();
    this._goldWindow.close();
    $gameMessage.clear();
};

Window_Message.prototype.updateLoading = function() {
    if (this._faceBitmap) {
        if (ImageManager.isReady()) {
            this.drawMessageFace();
            this._faceBitmap = null;
            return false;
        } else {
            return true;
        }
    } else if (!this._name == '') {
        if (ImageManager.isReady()) {
            this.drawNameLine();
    		return false;
        } else {
            return true;
        }    	
    } else {
        return false;
    }
};

Window_Message.prototype.drawNameLine = function() {
    var bitmap = ImageManager.loadSystem('NameLine');
    this.contents.blt(bitmap, 0, 0, 334, 17, 0, 36);
};

Window_Message.prototype.showBackgroundDimmer = function() {
    if (!this._dimmerSprite) {
        this._dimmerSprite = new Sprite();
        this._dimmerSprite.bitmap = ImageManager.loadSystem('MessageWindow');
        this.addChildToBack(this._dimmerSprite);
    }
    var bitmap = this._dimmerSprite.bitmap;
    if (bitmap.width !== this.width || bitmap.height !== this.height) {
        this.refreshDimmerBitmap();
    }
    this._dimmerSprite.visible = true;
    this.updateBackgroundDimmer();
};


//-----------------------------------------------------------------------------
// Window_ChoiceList
// this._messageWindow._choiceWindow.updateCursor();

Window_ChoiceList.prototype.initialize = function(messageWindow) {
    this._messageWindow = messageWindow;
    Window_Command.prototype.initialize.call(this, 0, 0);
    this.openness = 0;
    this.deactivate();
    this._background = 0;
};
//

Window_ChoiceList.prototype.updatePlacement = function() {	
    var positionType = $gameMessage.choicePositionType();
    var messageY = this._messageWindow.y;
    this.width = this.windowWidth();
    this.height = this.windowHeight();
    switch (positionType) {
    case 0:
        this.x = 60;
        break;
    case 1:
        this.x = (Graphics.boxWidth - this.width) / 2;
        break;
    case 2:
        this.x = Graphics.boxWidth - this.width - 60;
        break;
    }
    
    if (this._messageWindow._positionType == 0) {
        this.y = (Graphics.boxHeight / 2) - (this.height / 2) + 24;
    } else if (this._messageWindow._positionType == 1) {
        this.y = messageY + this._messageWindow.height + 24;
    } else {
        this.y = (Graphics.boxHeight / 2) - (this.height / 2) - 24;
    }
};

Window_ChoiceList.prototype.choiceItemRect = function(index) {
    var rect = new Rectangle();
    var maxCols = this.maxCols();
    rect.width = 300;
    rect.height = 56;
    rect.x = index % maxCols * (rect.width + this.spacing()) - this._scrollX;
    rect.y = Math.floor(index / maxCols) * rect.height + (index * 14) - this._scrollY;
    return rect;
};

Window_ChoiceList.prototype.itemRectForText = function(index) {
    var rect = this.itemRect(index);
    rect.x += this.textPadding();
    rect.width -= this.textPadding() * 2;
    return rect;
};

Window_ChoiceList.prototype.windowWidth = function() {
    return 300;
};

Window_ChoiceList.prototype.windowHeight = function() {
    return (this.numVisibleRows() * 56) + ((this.numVisibleRows() - 1) * 14);
};

Window_ChoiceList.prototype.contentsHeight = function() {
    return ((this.maxItems() * 56) + (this.maxItems() - 1) * 14);
};

Window_ChoiceList.prototype.drawItem = function(index) {
    var rect = this.itemRectForText(index);
    var rect = this.choiceItemRect(index);
    
    //var bitmap = ImageManager.loadSystem('SelectWindow');
    //this.contents.blt(bitmap, 0, 0, 300, 56, rect.x, rect.y);
    
    this.contents.fillStyle = 'rgb(0, 0, 0)';
    this.contents.fillRect(rect.x, rect.y, 300, 56);
    
    this.drawTextEx(this.commandName(index), rect.x + 24, rect.y + 11);
};


Window_ChoiceList.prototype.updateCursor = function() {
    this.refresh();
    if (this._cursorAll) {
        var allRowsHeight = this.maxRows() * this.itemHeight();
        this.setCursorRect(0, 0, this.contents.width, allRowsHeight);
        this.setTopRow(0);
    } else if (this.isCursorVisible()) {
        var rect = this.choiceItemRect(this.index());
    	var bitmap = ImageManager.loadSystem('SelectWindowCursor');
    	this.contents.blt(bitmap, 0, 0, 300, 56, rect.x, rect.y);
    //    this.setCursorRect(rect.x, rect.y, rect.width, rect.height);
    } else {
        this.setCursorRect(0, 0, 0, 0);
    }
};

Window_ChoiceList.prototype.hitTest = function(x, y) {
    if (this.isContentsArea(x, y)) {
        var cx = x - this.padding;
        var cy = y - this.padding;
        var topIndex = this.topIndex();
        for (var i = 0; i < this.maxPageItems(); i++) {
            var index = topIndex + i;
            if (index < this.maxItems()) {
                var rect = this.choiceItemRect(index);
                var right = rect.x + rect.width;
                var bottom = rect.y + rect.height;
                if (cx >= rect.x && cy >= rect.y && cx < right && cy < bottom) {
                    return index;
                }
            }
        }
    }
    return -1;
};


//-----------------------------------------------------------------------------
// Window_Collection

function Window_Collection() {
    this.initialize.apply(this, arguments);
}

Window_Collection.prototype = Object.create(Window_Base.prototype);
Window_Collection.prototype.constructor = Window_Collection;

Window_Collection.prototype.initialize = function() {
    Window_Base.prototype.initialize.call(this, 334, 154, 430, 330);
    this.visible = false;
    this.opacity = 0;
    this.refresh(0);
};

Window_Collection.prototype.standardFontSize = function() {
    return 18;
}

Window_Collection.prototype.refresh = function(itemno) {
    this.contents.clear();
    this.drawTextEx(this.value(itemno), 0, 0);
};

Window_Collection.prototype.value = function(itemno) {
	if (itemno == 0) {
		return "";
	} else {
		return autoNewLine($dataItems[itemno].note, 23);
	}
};
function autoNewLine(text, wordcount) {
	var result = "";
	var arr = text.split(/\r\n|\r|\n/);
	
    for (i = 0; i < arr.length; i++) {
		while (arr[i].length > wordcount) {
			if (arr[i].substr(wordcount,1) == "、" || arr[i].substr(wordcount,1) == "。") {
    			result += arr[i].substr(0,wordcount+1) + "\n";
    			arr[i] = arr[i].slice(wordcount+1);
			} else {
    			result += arr[i].substr(0,wordcount) + "\n";
    			arr[i] = arr[i].slice(wordcount);
    		}
    	}
    	result += arr[i] + "\n";
	}
	
	return result;
}


//-----------------------------------------------------------------------------
// rpg_scene.js > Scene_Map
//

Scene_Map.prototype.createDisplayObjects = function() {
    this.createSpriteset();
    this.createMapNameWindow();
    this.createCollectionWindow();
    this.createWindowLayer();
    this.createAllWindows();
};

Scene_Map.prototype.createCollectionWindow = function() {
    $collectionWindow = new Window_Collection();
    this.addChild($collectionWindow);
};


//-----------------------------------------------------------------------------
// rpg_objects.js > Game_Interpreter
//

Game_Interpreter.prototype.command101 = function() {
    if (!$gameMessage.isBusy()) {
    	$novel_mode = false;
        $gameMessage.setFaceImage(this._params[0], this._params[1]);
        $gameMessage.setBackground(this._params[2]);
        $gameMessage.setPositionType(this._params[3]);
        while (this.nextEventCode() === 401) {  // Text data
            this._index++;
            $gameMessage.add(this.currentCommand().parameters[0]);
        }
        switch (this.nextEventCode()) {
        case 102:  // Show Choices
            this._index++;
            this.setupChoices(this.currentCommand().parameters);
            break;
        case 103:  // Input Number
            this._index++;
            this.setupNumInput(this.currentCommand().parameters);
            break;
        case 104:  // Select Item
            this._index++;
            this.setupItemChoice(this.currentCommand().parameters);
            break;
        }
        this._index++;
        this.setWaitMode('message');
    }
    return false;
};

Game_Interpreter.prototype.command105 = function() {
    if (!$gameMessage.isBusy()) {
    	$novel_mode = true;
        $gameMessage._scrollMode = false;
        $gameMessage._background = 2;
        $gameMessage._positionType = 0;
        
        //$gameMessage.add('\\{');
        while (this.nextEventCode() === 405) {
            this._index++;
            var text = this.currentCommand().parameters[0];
        	
        	while (text.length > 25) {
        		if (text.substr(25,1) == "、" || text.substr(25,1) == "。" || text.substr(25,1) == "」") {
        			$gameMessage.add('\\>\\{' + text.substr(0,26) + '\\}');
        			text = text.slice(26);
        		} else {
        			$gameMessage.add('\\>\\{' + text.substr(0,25) + '\\}');
        			text = text.slice(25);
        		}
        	}
        	if (text != '') {
            	$gameMessage.add('\\>\\{' + text + '\\}');
            }
        }
        this._index++;
        this.setWaitMode('message');
    }
    return false;
};


/*
Window_Message.prototype.updateInput = function() {
    if (this.isAnySubWindowActive()) {
        return true;
    }
    if (this.pause) {
        if (Input.isRepeated('ok') || TouchInput.isRepeated()) {
            Input.update();
            this.pause = false;
            if (!this._textState) {
                this.terminateMessage();
            }
        } else if ($novel_mode && !($log_mode) && Input.isRepeated('cancel') || TouchInput.isCancelled()) {
        	$log_mode = true;
        	this.terminateMessage();
        }
        return true;
    }
    return false;
};
*/

//-----------------------------------------------------------------------------
// rpg_core.js >
//

Window.prototype.initialize = function() {
    PIXI.Container.call(this);

    this._isWindow = true;
    this._windowskin = null;
    this._width = 0;
    this._height = 0;
    this._cursorRect = new Rectangle();
    this._openness = 255;
    this._animationCount = 0;

    this._padding = 18;
    this._margin = 0;
    this._colorTone = [0, 0, 0];

    this._windowSpriteContainer = null;
    this._windowBackSprite = null;
    this._windowCursorSprite = null;
    this._windowFrameSprite = null;
    this._windowContentsSprite = null;
    this._windowArrowSprites = [];
    this._windowPauseSignSprite = null;

    this._createAllParts();

    /**
     * The origin point of the window for scrolling.
     *
     * @property origin
     * @type Point
     */
    this.origin = new Point();

    /**
     * The active state for the window.
     *
     * @property active
     * @type Boolean
     */
    this.active = true;

    /**
     * The visibility of the down scroll arrow.
     *
     * @property downArrowVisible
     * @type Boolean
     */
    this.downArrowVisible = false;

    /**
     * The visibility of the up scroll arrow.
     *
     * @property upArrowVisible
     * @type Boolean
     */
    this.upArrowVisible = false;

    /**
     * The visibility of the pause sign.
     *
     * @property pause
     * @type Boolean
     */
    this.pause = false;
};

Bitmap.prototype.drawText = function(text, x, y, maxWidth, lineHeight, align) {
    // Note: Firefox has a bug with textBaseline: Bug 737852
    //       So we use 'alphabetic' here.
    if (text !== undefined) {
        var tx = x;
        var ty = y + lineHeight - (lineHeight - this.fontSize * 0.7) / 2;
        var context = this._context;
        var alpha = context.globalAlpha;
        maxWidth = maxWidth || 0xffffffff;
        if (align === 'center') {
            tx += maxWidth / 2;
        }
        if (align === 'right') {
            tx += maxWidth;
        }
        context.save();
        context.font = this._makeFontNameText();
        context.textAlign = align;
        context.textBaseline = 'alphabetic';
        context.globalAlpha = 1;
        //this._drawTextOutline(text, tx, ty, maxWidth);
        context.globalAlpha = alpha;
        this._drawTextBody(text, tx, ty, maxWidth);
        context.restore();
        this._setDirty();
    }
};